@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import division, absolute_import, unicode_literals
 
-__version__ = '0.5.2'
+__version__ = '0.6.1'
 
 __author__ = 'Daniel Greenfeld & Contributors'
 
@@ -16,4 +16,11 @@ from . import types
 
 default = core.Admin2()
 ModelAdmin2 = types.ModelAdmin2
-Admin2Inline = types.Admin2Inline
+Admin2TabularInline = types.Admin2TabularInline
+Admin2StackedInline = types.Admin2StackedInline
+
+# Utility to make migration between versions easier
+sites = default
+ModelAdmin = ModelAdmin2
+AdminInline = Admin2TabularInline
+Admin2Inline = Admin2TabularInline
