@@ -1,6 +1,10 @@
+from django.db import models
 from django.test import TestCase
+from django.views.generic import View
 
-from ..types import immutable_admin_factory
+from .. import views
+from ..types import ModelAdmin2, immutable_admin_factory
+from ..core import Admin2
 
 
 class ModelAdmin(object):
@@ -34,3 +38,29 @@ class ImmutableAdminFactoryTests(TestCase):
         with self.assertRaises(AttributeError):
             # 'ImmutableAdmin' object has no attribute 'd'
             self.immutable_admin.d
+
+
+class Thing(models.Model):
+    pass
+
+
+class ModelAdminTest(TestCase):
+
+    def setUp(self):
+        class MyModelAdmin(ModelAdmin2):
+            my_view = views.AdminView(r'^$', views.ModelListView)
+
+        self.model_admin = MyModelAdmin
+
+    def test_views(self):
+        self.assertIn(
+            self.model_admin.my_view,
+            self.model_admin.views
+        )
+
+    def test_get_index_kwargs(self):
+        admin_instance = ModelAdmin2(Thing, Admin2)
+        self.assertIn(
+            'paginate_by',
+            admin_instance.get_index_kwargs().keys()
+        )
