@@ -3,9 +3,12 @@ from __future__ import division, absolute_import, unicode_literals
 
 from collections import namedtuple
 import logging
+import os
+import sys
 
 from django.core.urlresolvers import reverse
 from django.conf.urls import patterns, url
+from django.utils.six import with_metaclass
 
 import extra_views
 
@@ -20,7 +23,24 @@ from .forms import modelform_factory
 logger = logging.getLogger('djadmin2')
 
 
-class ModelAdmin2(object):
+class ModelAdminBase2(type):
+
+    def __new__(cls, name, bases, attrs):
+        new_class = super(ModelAdminBase2, cls).__new__(cls, name,
+                                                        bases, attrs)
+        view_list = getattr(new_class, 'views', [])
+
+        for key, value in attrs.items():
+            if isinstance(value, views.AdminView):
+                if not value.name:
+                    value.name = key
+                view_list.append(value)
+
+        setattr(new_class, 'views', view_list)
+        return new_class
+
+
+class ModelAdmin2(with_metaclass(ModelAdminBase2)):
     """
     Adding new ModelAdmin2 attributes:
 
@@ -39,7 +59,8 @@ class ModelAdmin2(object):
             This prevents us from easily implementing methods/setters which
             bypass the blocking features of the ImmutableAdmin.
     """
-
+    actions_selection_counter = True
+    date_hierarchy = False
     list_display = ('__str__',)
     list_display_links = ()
     list_filter = ()
@@ -53,6 +74,7 @@ class ModelAdmin2(object):
     verbose_name = None
     verbose_name_plural = None
     model_admin_attributes = settings.MODEL_ADMIN_ATTRS
+    ordering = False
     save_on_top = False
     save_on_bottom = True
 
@@ -96,11 +118,13 @@ class ModelAdmin2(object):
     inlines = []
 
     #  Views
-    index_view = views.ModelListView
-    create_view = views.ModelAddFormView
-    update_view = views.ModelEditFormView
-    detail_view = views.ModelDetailView
-    delete_view = views.ModelDeleteView
+    index_view = views.AdminView(r'^$', views.ModelListView, name='index')
+    create_view = views.AdminView(r'^create/$', views.ModelAddFormView, name='create')
+    update_view = views.AdminView(r'^(?P<pk>[0-9]+)/$', views.ModelEditFormView, name='update')
+    detail_view = views.AdminView(r'^(?P<pk>[0-9]+)/update/$', views.ModelDetailView, name='detail')
+    delete_view = views.AdminView(r'^(?P<pk>[0-9]+)/delete/$', views.ModelDeleteView, name='delete')
+    history_view = views.AdminView(r'^(?P<pk>[0-9]+)/history/$', views.ModelHistoryView, name='history')
+    views = []
 
     # API configuration
     api_serializer_class = None
@@ -133,6 +157,13 @@ class ModelAdmin2(object):
             'model_admin': immutable_admin_factory(self),
         }
 
+    def get_index_kwargs(self):
+        kwargs = self.get_default_view_kwargs()
+        kwargs.update({
+            'paginate_by': self.list_per_page,
+        })
+        return kwargs
+
     def get_default_api_view_kwargs(self):
         kwargs = self.get_default_view_kwargs()
         kwargs.update({
@@ -143,20 +174,19 @@ class ModelAdmin2(object):
     def get_prefixed_view_name(self, view_name):
         return '{}_{}'.format(self.name, view_name)
 
-    def get_index_kwargs(self):
-        return self.get_default_view_kwargs()
-
     def get_create_kwargs(self):
         kwargs = self.get_default_view_kwargs()
         kwargs.update({
             'inlines': self.inlines,
-            'form_class': self.create_form_class if self.create_form_class else self.form_class,
+            'form_class': (self.create_form_class if
+                           self.create_form_class else self.form_class),
         })
         return kwargs
 
     def get_update_kwargs(self):
         kwargs = self.get_default_view_kwargs()
-        form_class = self.update_form_class if self.update_form_class else self.form_class
+        form_class = (self.update_form_class if
+                      self.update_form_class else self.form_class)
         if form_class is None:
             form_class = modelform_factory(self.model)
         kwargs.update({
@@ -165,14 +195,9 @@ class ModelAdmin2(object):
         })
         return kwargs
 
-    def get_detail_kwargs(self):
-        return self.get_default_view_kwargs()
-
-    def get_delete_kwargs(self):
-        return self.get_default_view_kwargs()
-
     def get_index_url(self):
-        return reverse('admin2:{}'.format(self.get_prefixed_view_name('index')))
+        return reverse('admin2:{}'.format(
+            self.get_prefixed_view_name('index')))
 
     def get_api_list_kwargs(self):
         kwargs = self.get_default_api_view_kwargs()
@@ -185,36 +210,33 @@ class ModelAdmin2(object):
         return self.get_default_api_view_kwargs()
 
     def get_urls(self):
-        return patterns('',
-            url(
-                regex=r'^$',
-                view=self.index_view.as_view(**self.get_index_kwargs()),
-                name=self.get_prefixed_view_name('index')
-            ),
-            url(
-                regex=r'^create/$',
-                view=self.create_view.as_view(**self.get_create_kwargs()),
-                name=self.get_prefixed_view_name('create')
-            ),
-            url(
-                regex=r'^(?P<pk>[0-9]+)/$',
-                view=self.detail_view.as_view(**self.get_detail_kwargs()),
-                name=self.get_prefixed_view_name('detail')
-            ),
-            url(
-                regex=r'^(?P<pk>[0-9]+)/update/$',
-                view=self.update_view.as_view(**self.get_update_kwargs()),
-                name=self.get_prefixed_view_name('update')
-            ),
-            url(
-                regex=r'^(?P<pk>[0-9]+)/delete/$',
-                view=self.delete_view.as_view(**self.get_delete_kwargs()),
-                name=self.get_prefixed_view_name('delete')
-            ),
-        )
+        pattern_list = []
+        for admin_view in self.views:
+            admin_view.model_admin = self
+            get_kwargs = getattr(self, "get_%s_kwargs" % admin_view.name, None)
+            if not get_kwargs:
+                get_kwargs = admin_view.get_view_kwargs
+            try:
+                view_instance = admin_view.view.as_view(**get_kwargs())
+            except Exception as e:
+                trace = sys.exc_info()[2]
+                new_exception = TypeError(
+                    'Cannot instantiate admin view "{}.{}". '
+                    'The error that got raised was: {}'.format(
+                        self.__class__.__name__, admin_view.name, e))
+                raise new_exception, None, trace
+            pattern_list.append(
+                url(
+                    regex=admin_view.url,
+                    view=view_instance,
+                    name=self.get_prefixed_view_name(admin_view.name)
+                )
+            )
+        return patterns('', *pattern_list)
 
     def get_api_urls(self):
-        return patterns('',
+        return patterns(
+            '',
             url(
                 regex=r'^$',
                 view=self.api_list_view.as_view(**self.get_api_list_kwargs()),
@@ -222,7 +244,8 @@ class ModelAdmin2(object):
             ),
             url(
                 regex=r'^(?P<pk>[0-9]+)/$',
-                view=self.api_detail_view.as_view(**self.get_api_detail_kwargs()),
+                view=self.api_detail_view.as_view(
+                    **self.get_api_detail_kwargs()),
                 name=self.get_prefixed_view_name('api_detail'),
             ),
         )
@@ -243,18 +266,22 @@ class ModelAdmin2(object):
             class_actions = getattr(cls, 'list_actions', [])
             for action in class_actions:
                 actions_dict[action.__name__] = {
-                        'name': action.__name__,
-                        'description': actions.get_description(action),
-                        'action_callable': action
+                    'name': action.__name__,
+                    'description': actions.get_description(action),
+                    'action_callable': action
                 }
         return actions_dict
 
+    def get_ordering(self, request):
+        return self.ordering
+
 
 class Admin2Inline(extra_views.InlineFormSet):
     """
     A simple extension of django-extra-view's InlineFormSet that
     adds some useful functionality.
     """
+    template = None
 
     def construct_formset(self):
         """
@@ -263,18 +290,33 @@ class Admin2Inline(extra_views.InlineFormSet):
         """
         formset = super(Admin2Inline, self).construct_formset()
         formset.model = self.inline_model
+        formset.template = self.template
         return formset
 
 
-def immutable_admin_factory(model_admin):
-    """ Provide an ImmutableAdmin to make it harder for developers to dig themselves into holes.
-        See https://github.com/twoscoops/django-admin2/issues/99
-        Frozen class implementation as namedtuple suggested by Audrey Roy
+class Admin2TabularInline(Admin2Inline):
+    template = os.path.join(
+        settings.ADMIN2_THEME_DIRECTORY, 'edit_inlines/tabular.html')
 
-        Note: This won't stop developers from saving mutable objects to the result, but hopefully
-                developers attempting that 'workaround/hack' will read our documentation.
+
+class Admin2StackedInline(Admin2Inline):
+    template = os.path.join(
+        settings.ADMIN2_THEME_DIRECTORY, 'edit_inlines/stacked.html')
+
+
+def immutable_admin_factory(model_admin):
+    """
+    Provide an ImmutableAdmin to make it harder for developers to
+    dig themselves into holes.
+    See https://github.com/twoscoops/django-admin2/issues/99
+    Frozen class implementation as namedtuple suggested by Audrey Roy
+
+    Note: This won't stop developers from saving mutable objects to
+    the result, but hopefully developers attempting that
+    'workaround/hack' will read our documentation.
     """
     ImmutableAdmin = namedtuple('ImmutableAdmin',
                                 model_admin.model_admin_attributes,
                                 verbose=False)
-    return ImmutableAdmin(*[getattr(model_admin, x) for x in model_admin.model_admin_attributes])
+    return ImmutableAdmin(*[getattr(
+        model_admin, x) for x in model_admin.model_admin_attributes])
